@@ -1,5 +1,4 @@
 from .catalogue import Catalogue
-from .evaluator import Evaluator, OutputFormat
 from .exceptions import CatalogueError, JSONSchemaError, JSONPointerError, URIError
 from .json import JSON
 from .jsonpointer import JSONPointer
@@ -9,15 +8,13 @@ from .uri import URI
 __all__ = [
     'Catalogue',
     'CatalogueError',
-    'Evaluator',
     'JSON',
     'JSONPointer',
     'JSONPointerError',
     'JSONSchema',
     'JSONSchemaError',
-    'OutputFormat',
     'URI',
     'URIError',
 ]
 
-__version__ = '0.4.0'
+__version__ = '0.5.0'
