@@ -2,7 +2,8 @@ from typing import Callable
 
 from jschon.exceptions import CatalogueError
 from jschon.json import AnyJSONCompatible, JSON
-from jschon.jsonschema import Keyword, JSONSchema, Scope
+from jschon.jsonschema import JSONSchema, Scope
+from jschon.vocabulary import Keyword
 
 __all__ = [
     'FormatKeyword',
@@ -10,7 +11,7 @@ __all__ = [
 ]
 
 FormatValidator = Callable[[AnyJSONCompatible], None]
-"""The type of a ``"format"`` keyword validation function.
+"""The type of a ``"format"`` keyword validator.
 
 A :class:`~jschon.vocabulary.format.FormatValidator` is a
 callable accepting a JSON-compatible Python object as its
